@@ -139,7 +139,7 @@ class Uploader:
         dic = meta.copy()
         pypi_action = action
         dic[":action"] = pypi_action
-        dic["protocol_version"] = "1",
+        dic["protocol_version"] = "1"
         headers = {}
         auth = hub.current.get_auth()
         if auth:
@@ -405,10 +405,12 @@ class Exported:
 
     def setup_build(self, default_formats=None):
         deprecated_formats = []
+        sdist = self.args.sdist
+        wheel = self.args.wheel
         formats = self.args.formats
         if formats is None:
             formats = default_formats
-        if formats:
+        if formats and not sdist and not wheel:
             sdist = None
             wheel = None
             for format in formats.split(","):
@@ -447,9 +449,6 @@ class Exported:
                     "The --formats option is deprecated, "
                     "you can remove it to get the default sdist and wheel "
                     "releases you get with your currently specified formats.")
-        else:
-            sdist = self.args.sdist
-            wheel = self.args.wheel
 
         cmds = []
         if sdist is not None or wheel is not None:
@@ -559,6 +558,7 @@ def read_setupcfg(hub, path):
     setup_cfg = path.join("setup.cfg")
     if setup_cfg.exists():
         cfg = iniconfig.IniConfig(setup_cfg)
-        hub.line("detected devpi:upload section in %s" % setup_cfg, bold=True)
-        return cfg.sections.get("devpi:upload", {})
+        if 'devpi:upload' in cfg.sections:
+            hub.line("detected devpi:upload section in %s" % setup_cfg, bold=True)
+            return cfg.sections["devpi:upload"]
     return {}
