@@ -4,15 +4,21 @@ from devpi_common.metadata import parse_version
 import pytest
 
 try:
-    from urllib import quote as url_quote
-except ImportError:
     from urllib.parse import quote as url_quote
+except ImportError:
+    from urllib import quote as url_quote  # type: ignore
 
 
 class API:
     def __init__(self, d):
         self.__dict__ = d
 
+    def __repr__(self):
+        cls = self.__class__
+        name = "%s.%s" % (cls.__module__, cls.__name__)
+        return "<%s %r>" % (
+            name, self.__dict__.get('stagename', 'uninitialized'))
+
 
 class MappMixin:
     _usercount = 0
