@@ -96,7 +96,7 @@ class TestCheckout:
             runproc("git commit -m message")
         return repo
 
-    def test_vcs_export(self, uploadhub, repo, setupdir, tmpdir, monkeypatch):
+    def test_vcs_export(self, uploadhub, repo, setupdir, tmpdir):
         checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         assert checkout.rootpath == repo
         newrepo = tmpdir.mkdir("newrepo")
@@ -145,8 +145,7 @@ class TestCheckout:
         exported = checkout.export(tmpdir)
         assert exported.rootpath == checkout.setupdir
 
-    def test_vcs_export_verify_setup(self, uploadhub, setupdir,
-                                          tmpdir, monkeypatch):
+    def test_vcs_export_verify_setup(self, uploadhub, setupdir, tmpdir):
         subdir = setupdir.mkdir("subdir")
         subdir.ensure("setup.py")
         checkout = Checkout(uploadhub, uploadhub.args, subdir)
@@ -154,7 +153,7 @@ class TestCheckout:
         exported = checkout.export(wc)
         assert not exported.rootpath.join("setup.py").check()
 
-    def test_export_attributes(self, uploadhub, setupdir, tmpdir, monkeypatch):
+    def test_export_attributes(self, uploadhub, setupdir, tmpdir):
         checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         setupdir.join("setup.py").write(dedent("""
             from setuptools import setup
@@ -167,7 +166,7 @@ class TestCheckout:
         assert name == "xyz"
         assert version == "1.2.3"
 
-    def test_setup_build_docs(self, uploadhub, setupdir, tmpdir, monkeypatch):
+    def test_setup_build_docs(self, uploadhub, setupdir, tmpdir):
         checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         setupdir.join("setup.py").write(dedent("""
             from setuptools import setup
@@ -229,7 +228,7 @@ def test_parent_subpath(tmpdir):
 
 
 @pytest.mark.skipif("config.option.fast")
-def test_post_includes_auth_info(initproj, monkeypatch, uploadhub):
+def test_post_includes_auth_info(initproj, uploadhub):
     class Session:
         posts = []
 
@@ -241,7 +240,9 @@ def test_post_includes_auth_info(initproj, monkeypatch, uploadhub):
 
     class args:
         dryrun = None
-        formats = "sdist,bdist_wheel"
+        sdist = False
+        wheel = False
+        formats = None
         index = None
         no_isolation = True
         novcs = None
@@ -277,9 +278,63 @@ def test_post_includes_auth_info(initproj, monkeypatch, uploadhub):
     assert "X-Devpi-Auth" in upload2[1]["headers"]
 
 
+@pytest.mark.skipif("sys.version_info < (3,)")
+@pytest.mark.skipif("config.option.fast")
+def test_post_data(initproj, monkeypatch, reqmock, uploadhub):
+    import email
+
+    class args:
+        dryrun = None
+        sdist = False
+        wheel = False
+        formats = None
+        index = None
+        no_isolation = True
+        novcs = None
+        only_latest = None
+        onlydocs = None
+        path = None
+        python = None
+        setupdironly = None
+        verbose = 0
+        withdocs = None
+
+    class Response:
+        status_code = 200
+
+    sent = []
+
+    def send(req, **kw):
+        sent.append((req, kw))
+        return Response()
+
+    initproj("pkg-1.0")
+    tmpdir = py.path.local()
+    uploadhub.cwd = tmpdir
+    uploadhub.current.reconfigure(dict(
+        index="http://devpi/foo/bar",
+        login="http://devpi/+login",
+        pypisubmit="http://devpi/foo/bar"))
+    monkeypatch.setattr(uploadhub.http, "send", send)
+    main(uploadhub, args)
+    # convert POST data to Message
+    msg = email.message_from_bytes(
+        b"MIME-Version: 1.0\nContent-Type: %s\n\n%s" % (
+            sent[0][0].headers['Content-Type'].encode('ascii'),
+            sent[0][0].body))
+    # get the data
+    data = {
+        x.get_param("name", header="Content-Disposition"): x.get_payload()
+        for x in msg.get_payload()}
+    assert data[":action"] == "file_upload"
+    assert data["name"] == "pkg"
+    assert data["protocol_version"] == "1"
+    assert data["version"] == "1.0"
+
+
 @pytest.mark.skipif("sys.version_info < (3, 7)")
 @pytest.mark.skipif("config.option.fast")
-def test_post_derived_devpi_token(initproj, monkeypatch, uploadhub):
+def test_post_derived_devpi_token(initproj, uploadhub):
     from base64 import b64decode
     import pypitoken
 
@@ -587,6 +642,27 @@ class TestUploadFunctional:
         out = out_devpi("upload", "--no-isolation", "--index", "%s/dev" % user, "--dry-run")
         out.stdout.fnmatch_lines_random("skipped: file_upload*to*/%s/dev*" % user)
 
+    @pytest.mark.parametrize("other_index", ["root/pypi", "/"])
+    def test_index_option_with_environment_relative(
+            self, devpi, initproj, monkeypatch, out_devpi,
+            other_index, projname_version):
+        initproj(projname_version.rsplit("-", 1), {"doc": {
+            "conf.py": "#nothing",
+            "contents.rst": "",
+            "index.html": "<html/>"}})
+        assert py.path.local("setup.py").check()
+        # remember username
+        out = out_devpi("use")
+        user = re.search(r'\(logged in as (.+?)\)', out.stdout.str()).group(1)
+
+        # go to other index
+        out = out_devpi("use", other_index)
+
+        monkeypatch.setenv("DEVPI_INDEX", "user/dev")
+        # --index option
+        out = out_devpi("upload", "--no-isolation", "--index", "%s/dev" % user, "--dry-run")
+        out.stdout.fnmatch_lines_random("skipped: file_upload*to*/%s/dev*" % user)
+
     def test_logout(self, capfd, devpi, out_devpi, projname_version):
         # logoff then upload
         out = out_devpi("logoff")
@@ -601,7 +677,7 @@ class TestUploadFunctional:
         assert isinstance(res.sysex, SystemExit)
         assert res.sysex.args == (1,)
 
-    def test_fromdir(self, initproj, devpi, out_devpi, runproc, monkeypatch):
+    def test_fromdir(self, initproj, devpi, out_devpi, runproc):
         initproj("hello-1.1", {"doc": {
             "conf.py": "",
             "index.html": "<html/>"}})
@@ -653,6 +729,36 @@ class TestUploadFunctional:
         assert links["releasefile"] == "%s.zip" % name_version_str
         assert links["doczip"] == "%s.doc.zip" % name_version_str
 
+    def test_cli_sdist_precedence(self, initproj, devpi, out_devpi):
+        initproj("pkg-1.0")
+        tmpdir = py.path.local()
+        tmpdir.join("setup.cfg").write(dedent("""
+            [devpi:upload]
+            formats=bdist_wheel,sdist.zip"""))
+        hub = devpi("upload", "--sdist", "--no-isolation")
+        url = hub.current.get_index_url().url + 'pkg/1.0/'
+        out = out_devpi("getjson", url)
+        data = json.loads(out.stdout.str())
+        vv = ViewLinkStore(url, data["result"])
+        assert len(vv.get_links()) == 1
+        assert vv.get_links()[0].basename in ('pkg-1.0.tar.gz', 'pkg-1.0.zip')
+
+    def test_cli_wheel_precedence(self, initproj, devpi, out_devpi):
+        initproj("pkg-1.0")
+        tmpdir = py.path.local()
+        tmpdir.join("setup.cfg").write(dedent("""
+            [devpi:upload]
+            formats=bdist_wheel,sdist.zip"""))
+        hub = devpi("upload", "--wheel", "--no-isolation")
+        url = hub.current.get_index_url().url + 'pkg/1.0/'
+        out = out_devpi("getjson", url)
+        data = json.loads(out.stdout.str())
+        vv = ViewLinkStore(url, data["result"])
+        assert len(vv.get_links()) == 1
+        assert vv.get_links()[0].basename in (
+            'pkg-1.0-py2-none-any.whl',
+            'pkg-1.0-py3-none-any.whl')
+
 
 def test_getpkginfo(datadir):
     info = get_pkginfo(datadir.join("dddttt-0.1.dev45-py27-none-any.whl"))
